@@ -40,6 +40,7 @@
 #include "got_privsep.h"
 
 #include "gotwebd.h"
+#include "log.h"
 
 static const struct got_error *got_init_repo_commit(struct repo_commit **);
 static const struct got_error *got_init_repo_tag(struct repo_tag **);
@@ -59,7 +60,7 @@ got_gotweb_closefile(FILE *f)
 	if (fseek(f, 0, SEEK_SET) == -1)
 		err = got_error_from_errno("fseek");
 
-	if (err == NULL && ftruncate(fileno(f), 0) == -1)
+	if (ftruncate(fileno(f), 0) == -1 && err == NULL)
 		err = got_error_from_errno("ftruncate");
 
 	if (fclose(f) == EOF && err == NULL)
@@ -100,26 +101,18 @@ got_gotweb_dupfd(int *priv_fd, int *fd)
 const struct got_error *
 got_get_repo_owner(char **owner, struct request *c)
 {
-	struct server *srv = c->srv;
 	struct transport *t = c->t;
 	struct got_repository *repo = t->repo;
 	const char *gitconfig_owner;
 
 	*owner = NULL;
-
-	if (srv->show_repo_owner == 0)
-		return NULL;
-
 	gitconfig_owner = got_repo_get_gitconfig_owner(repo);
 	if (gitconfig_owner) {
 		*owner = strdup(gitconfig_owner);
 		if (*owner == NULL)
 			return got_error_from_errno("strdup");
-	} else {
-		*owner = strdup("");
-		if (*owner == NULL)
-			return got_error_from_errno("strdup");
 	}
+
 	return NULL;
 }
 
@@ -1152,15 +1145,15 @@ got_open_diff_for_output(FILE **fp, struct request *c)
 
 	error = got_gotweb_openfile(&f1, &c->priv_fd[DIFF_FD_1]);
 	if (error)
-		return error;
+		goto done;
 
 	error = got_gotweb_openfile(&f2, &c->priv_fd[DIFF_FD_2]);
 	if (error)
-		return error;
+		goto done;
 
 	error = got_gotweb_openfile(&f3, &c->priv_fd[DIFF_FD_3]);
 	if (error)
-		return error;
+		goto done;
 
 	rc = TAILQ_FIRST(&t->repo_commits);
 
