@@ -49,6 +49,7 @@
 #include "got_privsep.h"
 
 #include "gotwebd.h"
+#include "log.h"
 #include "tmpl.h"
 
 static const struct querystring_keys querystring_keys[] = {
@@ -79,13 +80,15 @@ static const struct action_keys action_keys[] = {
 };
 
 static const struct got_error *gotweb_init_querystring(struct querystring **);
-static const struct got_error *gotweb_parse_querystring(struct querystring **,
+static const struct got_error *gotweb_parse_querystring(struct querystring *,
     char *);
-static const struct got_error *gotweb_assign_querystring(struct querystring **,
+static const struct got_error *gotweb_assign_querystring(struct querystring *,
     char *, char *);
 static int gotweb_render_index(struct template *);
 static const struct got_error *gotweb_load_got_path(struct repo_dir **,
     const char *, struct request *);
+static const struct got_error *gotweb_load_file(char **, const char *,
+    const char *, int);
 static const struct got_error *gotweb_get_repo_description(char **,
     struct server *, const char *, int);
 static const struct got_error *gotweb_get_clone_url(char **, struct server *,
@@ -143,6 +146,7 @@ gotweb_process_request(struct request *c)
 	struct server *srv = NULL;
 	struct querystring *qs = NULL;
 	struct repo_dir *repo_dir = NULL;
+	struct repo_commit *commit;
 	const char *rss_ctype = "application/rss+xml;charset=utf-8";
 	const uint8_t *buf;
 	size_t len;
@@ -154,9 +158,6 @@ gotweb_process_request(struct request *c)
 		log_warnx("%s: %s", __func__, error->msg);
 		return;
 	}
-	/* don't process any further if client disconnected */
-	if (c->sock->client_status == CLIENT_DISCONNECT)
-		return;
 	/* get the gotwebd server */
 	srv = gotweb_get_server(c->server_name);
 	if (srv == NULL) {
@@ -171,7 +172,7 @@ gotweb_process_request(struct request *c)
 		goto err;
 	}
 	c->t->qs = qs;
-	error = gotweb_parse_querystring(&qs, c->querystring);
+	error = gotweb_parse_querystring(qs, c->querystring);
 	if (error) {
 		log_warnx("%s: %s", __func__, error->msg);
 		goto err;
@@ -195,7 +196,7 @@ gotweb_process_request(struct request *c)
 	if (qs->action != INDEX) {
 		error = gotweb_load_got_path(&repo_dir, qs->path, c);
 		c->t->repo_dir = repo_dir;
-		if (error && error->code != GOT_ERR_LONELY_PACKIDX)
+		if (error)
 			goto err;
 	}
 
@@ -352,6 +353,15 @@ gotweb_process_request(struct request *c)
 		error = got_get_repo_commits(c, srv->summary_commits_display);
 		if (error)
 			goto err;
+		commit = TAILQ_FIRST(&c->t->repo_commits);
+		if (commit && qs->commit == NULL) {
+			qs->commit = strdup(commit->commit_id);
+			if (qs->commit == NULL) {
+				error = got_error_from_errno("strdup");
+				log_warn("%s: strdup", __func__);
+				goto err;
+			}
+		}
 		qs->action = TAGS;
 		error = got_get_repo_tags(c, srv->summary_tags_display);
 		if (error) {
@@ -466,7 +476,7 @@ gotweb_init_querystring(struct querystring **qs)
 }
 
 static const struct got_error *
-gotweb_parse_querystring(struct querystring **qs, char *qst)
+gotweb_parse_querystring(struct querystring *qs, char *qst)
 {
 	const struct got_error *error = NULL;
 	char *tok1 = NULL, *tok1_pair = NULL, *tok1_end = NULL;
@@ -561,7 +571,7 @@ gotweb_urldecode(char *url)
 }
 
 static const struct got_error *
-gotweb_assign_querystring(struct querystring **qs, char *key, char *value)
+gotweb_assign_querystring(struct querystring *qs, char *key, char *value)
 {
 	const struct got_error *error = NULL;
 	const char *errstr;
@@ -580,44 +590,40 @@ gotweb_assign_querystring(struct querystring **qs, char *key, char *value)
 			for (a_cnt = 0; a_cnt < nitems(action_keys); a_cnt++) {
 				if (strcmp(value, action_keys[a_cnt].name) != 0)
 					continue;
-				else if (strcmp(value,
-				    action_keys[a_cnt].name) == 0){
-					(*qs)->action =
-					    action_keys[a_cnt].action;
-					goto qa_found;
-				}
+				qs->action = action_keys[a_cnt].action;
+				goto qa_found;
 			}
-			(*qs)->action = ERR;
+			qs->action = ERR;
 qa_found:
 			break;
 		case COMMIT:
-			(*qs)->commit = strdup(value);
-			if ((*qs)->commit == NULL) {
+			qs->commit = strdup(value);
+			if (qs->commit == NULL) {
 				error = got_error_from_errno2(__func__,
 				    "strdup");
 				goto done;
 			}
 			break;
 		case RFILE:
-			(*qs)->file = strdup(value);
-			if ((*qs)->file == NULL) {
+			qs->file = strdup(value);
+			if (qs->file == NULL) {
 				error = got_error_from_errno2(__func__,
 				    "strdup");
 				goto done;
 			}
 			break;
 		case FOLDER:
-			(*qs)->folder = strdup(value);
-			if ((*qs)->folder == NULL) {
+			qs->folder = strdup(value);
+			if (qs->folder == NULL) {
 				error = got_error_from_errno2(__func__,
 				    "strdup");
 				goto done;
 			}
 			break;
 		case HEADREF:
-			free((*qs)->headref);
-			(*qs)->headref = strdup(value);
-			if ((*qs)->headref == NULL) {
+			free(qs->headref);
+			qs->headref = strdup(value);
+			if (qs->headref == NULL) {
 				error = got_error_from_errno2(__func__,
 				    "strdup");
 				goto done;
@@ -626,19 +632,19 @@ qa_found:
 		case INDEX_PAGE:
 			if (*value == '\0')
 				break;
-			(*qs)->index_page = strtonum(value, INT64_MIN,
+			qs->index_page = strtonum(value, INT64_MIN,
 			    INT64_MAX, &errstr);
 			if (errstr) {
 				error = got_error_from_errno3(__func__,
 				    "strtonum", errstr);
 				goto done;
 			}
-			if ((*qs)->index_page < 0)
-				(*qs)->index_page = 0;
+			if (qs->index_page < 0)
+				qs->index_page = 0;
 			break;
 		case PATH:
-			(*qs)->path = strdup(value);
-			if ((*qs)->path == NULL) {
+			qs->path = strdup(value);
+			if (qs->path == NULL) {
 				error = got_error_from_errno2(__func__,
 				    "strdup");
 				goto done;
@@ -818,7 +824,7 @@ gotweb_render_index(struct template *tp)
 
 		error = gotweb_load_got_path(&repo_dir, sd_dent[d_i]->d_name,
 		    c);
-		if (error && error->code != GOT_ERR_LONELY_PACKIDX) {
+		if (error) {
 			if (error->code != GOT_ERR_NOT_GIT_REPO)
 				log_warnx("%s: %s: %s", __func__,
 				    sd_dent[d_i]->d_name, error->msg);
@@ -982,18 +988,6 @@ gotweb_render_url(struct request *c, struct gotweb_url *url)
 		sep = "&";
 	}
 
-	if (url->previd) {
-		if (tp_writef(c->tp, "%sprevid=%s", sep, url->previd) == -1)
-			return -1;
-		sep = "&";
-	}
-
-	if (url->prevset) {
-		if (tp_writef(c->tp, "%sprevset=%s", sep, url->prevset) == -1)
-			return -1;
-		sep = "&";
-	}
-
 	if (url->file) {
 		tmp = gotweb_urlencode(url->file);
 		if (tmp == NULL)
@@ -1068,7 +1062,6 @@ gotweb_load_got_path(struct repo_dir **rp, const char *dir,
     struct request *c)
 {
 	const struct got_error *error = NULL;
-	struct socket *sock = c->sock;
 	struct server *srv = c->srv;
 	struct transport *t = c->t;
 	struct repo_dir *repo_dir;
@@ -1119,16 +1112,25 @@ gotweb_load_got_path(struct repo_dir **rp, const char *dir,
 		goto err;
 	}
 
-	error = got_repo_open(&t->repo, repo_dir->path, NULL, sock->pack_fds);
+	error = got_repo_open(&t->repo, repo_dir->path, NULL,
+	    gotwebd_env->pack_fds);
 	if (error)
 		goto err;
 	error = gotweb_get_repo_description(&repo_dir->description, srv,
 	    repo_dir->path, dirfd(dt));
 	if (error)
 		goto err;
-	error = got_get_repo_owner(&repo_dir->owner, c);
-	if (error)
-		goto err;
+	if (srv->show_repo_owner) {
+		error = gotweb_load_file(&repo_dir->owner, repo_dir->path,
+		    "owner", dirfd(dt));
+		if (error)
+			goto err;
+		if (repo_dir->owner == NULL) {
+			error = got_get_repo_owner(&repo_dir->owner, c);
+			if (error)
+				goto err;
+		}
+	}
 	if (srv->show_repo_age) {
 		error = got_get_repo_age(&repo_dir->age, c, NULL);
 		if (error)
@@ -1148,30 +1150,24 @@ err:
 }
 
 static const struct got_error *
-gotweb_get_repo_description(char **description, struct server *srv,
-    const char *dirpath, int dir)
+gotweb_load_file(char **str, const char *dir, const char *file, int dirfd)
 {
 	const struct got_error *error = NULL;
 	struct stat sb;
-	int fd = -1;
 	off_t len;
+	int fd;
 
-	*description = NULL;
-	if (srv->show_repo_description == 0)
-		return NULL;
+	*str = NULL;
 
-	fd = openat(dir, "description", O_RDONLY);
+	fd = openat(dirfd, file, O_RDONLY);
 	if (fd == -1) {
-		if (errno != ENOENT && errno != EACCES) {
-			error = got_error_from_errno_fmt("openat %s/%s",
-			    dirpath, "description");
-		}
-		goto done;
+		if (errno == ENOENT || errno == EACCES)
+			return NULL;
+		return got_error_from_errno_fmt("openat %s/%s", dir, file);
 	}
 
 	if (fstat(fd, &sb) == -1) {
-		error = got_error_from_errno_fmt("fstat %s/%s",
-		    dirpath, "description");
+		error = got_error_from_errno_fmt("fstat %s/%s", dir, file);
 		goto done;
 	}
 
@@ -1179,64 +1175,40 @@ gotweb_get_repo_description(char **description, struct server *srv,
 	if (len > GOTWEBD_MAXDESCRSZ - 1)
 		len = GOTWEBD_MAXDESCRSZ - 1;
 
-	*description = calloc(len + 1, sizeof(**description));
-	if (*description == NULL) {
+	*str = calloc(len + 1, 1);
+	if (*str == NULL) {
 		error = got_error_from_errno("calloc");
 		goto done;
 	}
 
-	if (read(fd, *description, len) == -1)
+	if (read(fd, *str, len) == -1)
 		error = got_error_from_errno("read");
-done:
+ done:
 	if (fd != -1 && close(fd) == -1 && error == NULL)
 		error = got_error_from_errno("close");
 	return error;
 }
 
+static const struct got_error *
+gotweb_get_repo_description(char **description, struct server *srv,
+    const char *dirpath, int dir)
+{
+	*description = NULL;
+	if (srv->show_repo_description == 0)
+		return NULL;
+
+	return gotweb_load_file(description, dirpath, "description", dir);
+}
+
 static const struct got_error *
 gotweb_get_clone_url(char **url, struct server *srv, const char *dirpath,
     int dir)
 {
-	const struct got_error *error = NULL;
-	struct stat sb;
-	int fd = -1;
-	off_t len;
-
 	*url = NULL;
 	if (srv->show_repo_cloneurl == 0)
 		return NULL;
 
-	fd = openat(dir, "cloneurl", O_RDONLY);
-	if (fd == -1) {
-		if (errno != ENOENT && errno != EACCES) {
-			error = got_error_from_errno_fmt("openat %s/%s",
-			    dirpath, "cloneurl");
-		}
-		goto done;
-	}
-
-	if (fstat(fd, &sb) == -1) {
-		error = got_error_from_errno_fmt("fstat %s/%s",
-		    dirpath, "cloneurl");
-		goto done;
-	}
-
-	len = sb.st_size;
-	if (len > GOTWEBD_MAXCLONEURLSZ - 1)
-		len = GOTWEBD_MAXCLONEURLSZ - 1;
-
-	*url = calloc(len + 1, sizeof(**url));
-	if (*url == NULL) {
-		error = got_error_from_errno("calloc");
-		goto done;
-	}
-
-	if (read(fd, *url, len) == -1)
-		error = got_error_from_errno("read");
-done:
-	if (fd != -1 && close(fd) == -1 && error == NULL)
-		error = got_error_from_errno("close");
-	return error;
+	return gotweb_load_file(url, dirpath, "cloneurl", dir);
 }
 
 int
