@@ -936,7 +936,7 @@ url.index_page = -1;
 url.action = BLOB;
 url.path = t->qs->path;
 url.file = readme;
-url.folder = t->qs->folder;
+url.folder = t->qs->folder ? t->qs->folder : "";
 url.commit = t->qs->commit;
 #line 678 "../gotwebd/pages.tmpl"
 if (!binary) {
