@@ -59,7 +59,7 @@
 #define GOT_ERR_PACK_OFFSET	42
 #define GOT_ERR_OBJ_EXISTS	43
 #define GOT_ERR_BAD_OBJ_ID	44
-/* 45 is currently unused */
+#define GOT_ERR_OBJECT_FORMAT	45
 #define GOT_ERR_ITER_COMPLETED	46
 #define GOT_ERR_RANGE		47
 #define GOT_ERR_EXPECTED	48 /* for use in regress tests only */
@@ -187,6 +187,7 @@
 #define GOT_ERR_MERGE_COMMIT_OUT_OF_DATE 170
 #define GOT_ERR_BUNDLE_FORMAT 171
 #define GOT_ERR_BAD_KEYWORD	172
+#define GOT_ERR_UNKNOWN_CAPA	173
 
 struct got_error {
         int code;
