@@ -39,6 +39,10 @@ struct got_fileindex;
 #define GOT_STATUS_BASE_REF_ERR	'B'
 #define GOT_STATUS_CANNOT_UPDATE '#'
 
+/* Also defined in got_lib_worktree.h in case got_worktree.h is not included. */
+#define GOT_WORKTREE_GOT_DIR		".got"
+#define GOT_WORKTREE_CVG_DIR		".cvg"
+
 /*
  * Attempt to initialize a new work tree on disk.
  * The first argument is the path to a directory where the work tree
@@ -46,15 +50,21 @@ struct got_fileindex;
  * of the path must already exist.
  * The reference provided will be used to determine the new worktree's
  * base commit. The third argument speficies the work tree's path prefix.
+ * The fourth argument specifies the meta data directory to use, which
+ * should be either GOT_WORKTREE_GOT_DIR or GOT_WORKTREE_CVG_DIR.
  */
 const struct got_error *got_worktree_init(const char *, struct got_reference *,
-    const char *, struct got_repository *);
+    const char *, const char *, struct got_repository *);
 
 /*
- * Attempt to open a worktree at or above the specified path.
+ * Attempt to open a worktree at or above the specified path, using
+ * the specified meta data directory which should be either be NULL
+ * in which case a meta directory is auto-discovered, or be one of
+ * GOT_WORKTREE_GOT_DIR and GOT_WORKTREE_CVG_DIR.
  * The caller must dispose of it with got_worktree_close().
  */
-const struct got_error *got_worktree_open(struct got_worktree **, const char *);
+const struct got_error *got_worktree_open(struct got_worktree **,
+    const char *path, const char *meta_dir);
 
 /* Dispose of an open work tree. */
 const struct got_error *got_worktree_close(struct got_worktree *);
@@ -80,6 +90,11 @@ const char *got_worktree_get_path_prefix(struct got_worktree *);
  */
 const struct got_error *got_worktree_get_uuid(char **, struct got_worktree *);
 
+/*
+ * Get the work tree's format version number.
+ */
+int got_worktree_get_format_version(struct got_worktree *);
+
 /*
  * Check if a user-provided path prefix matches that of the worktree.
  */
@@ -123,6 +138,20 @@ struct got_object_id *got_worktree_get_base_commit_id(struct got_worktree *);
 const struct got_error *got_worktree_set_base_commit_id(struct got_worktree *,
     struct got_repository *, struct got_object_id *);
 
+/*
+ * Get the state of the work tree. If the work tree's global base commit is
+ * the tip of the work tree's current branch, and each file in the index is
+ * based on this same commit, the char out parameter will be
+ * GOT_WORKTREE_STATE_UPTODATE, else it will be GOT_WORKTREE_STATE_OUTOFDATE.
+ */
+const struct got_error *got_worktree_get_state(char *,
+    struct got_repository *, struct got_worktree *,
+    got_cancel_cb, void *);
+
+#define GOT_WORKTREE_STATE_UNKNOWN	' '
+#define GOT_WORKTREE_STATE_UPTODATE	'*'
+#define GOT_WORKTREE_STATE_OUTOFDATE	'~'
+
 /*
  * Obtain a parsed representation of this worktree's got.conf file.
  * Return NULL if this configuration file could not be read.
@@ -295,6 +324,9 @@ const struct got_error *got_worktree_rebase_continue(struct got_object_id **,
 /* Check whether a, potentially interrupted, rebase operation is in progress. */
 const struct got_error *got_worktree_rebase_in_progress(int *,
     struct got_worktree *);
+/* Return information about an in-progress rebase operation. */
+const struct got_error *got_worktree_rebase_info(char **new_base_branch_name,
+    char **branch_name, struct got_worktree *, struct got_repository *);
 
 /*
  * Merge changes from the commit currently being rebased into the work tree.
@@ -367,6 +399,10 @@ const struct got_error *got_worktree_histedit_continue(struct got_object_id **,
 /* Check whether a histedit operation is in progress. */
 const struct got_error *got_worktree_histedit_in_progress(int *,
     struct got_worktree *);
+/* Return information about an in-progress histedit operation. */
+const struct got_error *got_worktree_histedit_info(
+    char **branch_nane, struct got_worktree *,
+    struct got_repository *);
 
 /*
  * Merge changes from the commit currently being edited into the work tree.
@@ -483,6 +519,10 @@ const struct got_error *got_worktree_merge_complete(
 /* Check whether a merge operation is in progress. */
 const struct got_error *got_worktree_merge_in_progress(int *,
     struct got_worktree *, struct got_repository *);
+/* Return information about an in-progress merge operation. */
+const struct got_error *
+got_worktree_merge_info(char **branch_name, struct got_worktree *,
+    struct got_repository *);
 
 /*
  * Prepare for merging a branch into the work tree's current branch: lock the
@@ -543,6 +583,21 @@ const struct got_error *got_worktree_unstage(struct got_worktree *,
     struct got_pathlist_head *, got_worktree_checkout_cb, void *,
     got_worktree_patch_cb, void *, struct got_repository *);
 
+/*
+ * Prepare for getting meta data for paths in the work tree.  This
+ * function also returns a poniter to a fileindex wihch must be passed
+ * back to other path_info-related functions and *_version() functions.
+ */
+const struct got_error *
+got_worktree_path_info_prepare(struct got_fileindex **,
+    struct got_worktree *, struct got_repository *);
+
+/*
+ * Get the file-index version.
+ */
+uint32_t
+got_worktree_get_fileindex_version(struct got_fileindex *);
+
 /* A callback function which is invoked with per-path info. */
 typedef const struct got_error *(*got_worktree_path_info_cb)(void *,
     const char *path, mode_t mode, time_t mtime,
@@ -555,8 +610,15 @@ typedef const struct got_error *(*got_worktree_path_info_cb)(void *,
  * a path, and meta-data arguments (see got_worktree_path_info_cb).
  */
 const struct got_error *
-got_worktree_path_info(struct got_worktree *, struct got_pathlist_head *,
-    got_worktree_path_info_cb, void *, got_cancel_cb , void *);
+got_worktree_path_info(struct got_worktree *, struct got_fileindex *,
+    struct got_pathlist_head *, got_worktree_path_info_cb, void *,
+    got_cancel_cb , void *);
+
+/*
+ * Complete the current path_info operation.
+ */
+const struct got_error *
+got_worktree_path_info_complete(struct got_fileindex *, struct got_worktree *);
 
 /* References pointing at pre-rebase commit backups. */
 #define GOT_WORKTREE_REBASE_BACKUP_REF_PREFIX "refs/got/backup/rebase"
@@ -569,7 +631,7 @@ got_worktree_path_info(struct got_worktree *, struct got_pathlist_head *,
  */
 const struct got_error *
 got_worktree_patch_prepare(struct got_fileindex **, char **,
-    struct got_worktree *);
+    struct got_worktree *, struct got_repository *);
 
 /*
  * Lookup paths for the "old" and "new" file before patching and check their
@@ -591,4 +653,5 @@ got_worktree_patch_schedule_rm(const char *, struct got_repository *,
 
 /* Complete the patch operation. */
 const struct got_error *
-got_worktree_patch_complete(struct got_fileindex *, const char *);
+got_worktree_patch_complete(struct got_worktree *, struct got_fileindex *,
+    const char *);
