@@ -91,6 +91,7 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_PACK_OFFSET,	"bad offset in pack file" },
 	{ GOT_ERR_OBJ_EXISTS,	"object already exists" },
 	{ GOT_ERR_BAD_OBJ_ID,	"bad object id" },
+	{ GOT_ERR_OBJECT_FORMAT, "object format not supported" },
 	{ GOT_ERR_ITER_COMPLETED,"iteration completed" },
 	{ GOT_ERR_RANGE,	"value out of range" },
 	{ GOT_ERR_EXPECTED,	"expected an error but have no error" },
@@ -139,6 +140,7 @@ static const struct got_error got_errors[] = {
 	    "changes must be committed or reverted first" },
 	{ GOT_ERR_NOT_REBASING,	"rebase operation not in progress" },
 	{ GOT_ERR_REBASE_COMMITID,"rebase commit ID mismatch" },
+	{ GOT_ERR_WRONG_BRANCH, "update -b required" },
 	{ GOT_ERR_REBASING,	"a rebase operation is in progress in this "
 	    "work tree and must be continued or aborted first" },
 	{ GOT_ERR_REBASE_PATH,	"cannot rebase branch which contains "
@@ -199,7 +201,7 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_SEND_BAD_REF, "reference cannot be sent" },
 	{ GOT_ERR_SEND_FAILED, "could not send pack file" },
 	{ GOT_ERR_SEND_EMPTY, "no references to send" },
-	{ GOT_ERR_SEND_ANCESTRY, "fetch and rebase required" },
+	{ GOT_ERR_SEND_ANCESTRY, "branch on server has a different ancestry; either fetch changes from server and then rebase or merge local branch before sending, or ignore ancestry with send -f (can lead to data loss on server)" },
 	{ GOT_ERR_CAPA_DELETE_REFS, "server cannot delete references" },
 	{ GOT_ERR_SEND_DELETE_REF, "reference cannot be deleted" },
 	{ GOT_ERR_SEND_TAG_EXISTS, "tag already exists on server" },
@@ -243,7 +245,8 @@ static const struct got_error got_errors[] = {
 	    "the work tree is no longer up-to-date; merge must be aborted "
 	    "and retried" },
 	{ GOT_ERR_BUNDLE_FORMAT, "unknown git bundle version" },
-	{ GOT_ERR_BAD_KEYWORD, "invalid commit keyword" }
+	{ GOT_ERR_BAD_KEYWORD, "invalid commit keyword" },
+	{ GOT_ERR_UNKNOWN_CAPA, "unknown capability" },
 };
 
 static struct got_custom_error {
