@@ -260,7 +260,8 @@ struct got_imsg_commit_object {
 } __attribute__((__packed__));
 
 struct got_imsg_tree_entry {
-	char id[SHA1_DIGEST_LENGTH];
+	char id[GOT_HASH_DIGEST_MAXLEN];
+	int algo;
 	mode_t mode;
 	size_t namelen;
 	/* Followed by namelen bytes of entry's name, not NUL-terminated. */
@@ -354,7 +355,7 @@ struct got_imsg_commit_painting_request {
 
 /* Structure for GOT_IMSG_PAINTED_COMMITS. */
 struct got_imsg_painted_commit {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	intptr_t color;
 } __attribute__((__packed__));
 
@@ -491,7 +492,7 @@ struct got_imsg_send_ref_status {
 
 /* Structure for GOT_IMSG_IDXPACK_REQUEST data. */
 struct got_imsg_index_pack_request {
-	uint8_t pack_hash[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 } __attribute__((__packed__));
 
 /* Structure for GOT_IMSG_IDXPACK_PROGRESS data. */
@@ -513,6 +514,7 @@ struct got_imsg_index_pack_progress {
 struct got_imsg_packidx {
 	size_t len;
 	off_t packfile_size;
+	int algo;
 	/* Additionally, a file desciptor is passed via imsg. */
 };
 
@@ -520,6 +522,7 @@ struct got_imsg_packidx {
 struct got_imsg_pack {
 	char path_packfile[PATH_MAX];
 	off_t filesize;
+	int algo;
 	/* Additionally, a file desciptor is passed via imsg. */
 } __attribute__((__packed__));
 
@@ -592,13 +595,13 @@ struct got_imsg_traversed_commits {
 
 /* Structure for GOT_IMSG_ENUMERATED_COMMIT  */
 struct got_imsg_enumerated_commit {
-	uint8_t id[SHA1_DIGEST_LENGTH];
+	struct got_object_id id;
 	time_t mtime;
 } __attribute__((__packed__));
 
 /* Structure for GOT_IMSG_ENUMERATED_TREE */
 struct got_imsg_enumerated_tree {
-	uint8_t id[SHA1_DIGEST_LENGTH]; /* tree ID */
+	struct got_object_id id;	/* tree ID */
 	int nentries;			/* number of tree entries */
 
 	/* Followed by tree's path in remaining data of imsg buffer. */
@@ -652,8 +655,8 @@ struct got_imsg_patch {
 	int	xbit;
 	char	old[PATH_MAX];
 	char	new[PATH_MAX];
-	char	cid[41];
-	char	blob[41];
+	char	cid[GOT_HASH_DIGEST_STRING_MAXLEN];
+	char	blob[GOT_HASH_DIGEST_STRING_MAXLEN];
 };
 
 /*
@@ -698,7 +701,7 @@ const struct got_error *got_privsep_send_tmpfd(struct imsgbuf *, int);
 const struct got_error *got_privsep_send_obj(struct imsgbuf *,
     struct got_object *);
 const struct got_error *got_privsep_send_index_pack_req(struct imsgbuf *,
-    uint8_t *, int);
+    struct got_object_id *, int);
 const struct got_error *got_privsep_send_index_pack_outfd(struct imsgbuf *,
     int);
 const struct got_error *got_privsep_recv_index_progress(int *, int *, int *,
