@@ -14,6 +14,8 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "got_compat.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/queue.h>
@@ -31,8 +33,6 @@
 #include <limits.h>
 #include <imsg.h>
 
-#include "got_compat.h"
-
 #include "got_error.h"
 #include "got_object.h"
 #include "got_repository.h"
@@ -115,7 +115,7 @@ got_object_get_path(char **path, struct got_object_id *id,
 		goto done;
 
 	if (asprintf(path, "%s/%.2x/%s", path_objects,
-	    id->sha1[0], hex + 2) == -1)
+	    id->hash[0], hex + 2) == -1)
 		err = got_error_from_errno("asprintf");
 
 done:
@@ -150,7 +150,7 @@ got_object_open_by_id_str(struct got_object **obj, struct got_repository *repo,
 {
 	struct got_object_id id;
 
-	if (!got_parse_object_id(&id, id_str, GOT_HASH_SHA1))
+	if (!got_parse_object_id(&id, id_str, repo->algo))
 		return got_error_path(id_str, GOT_ERR_BAD_OBJ_ID_STR);
 
 	return got_object_open(obj, repo, &id);
@@ -175,41 +175,6 @@ got_object_resolve_id_str(struct got_object_id **id,
 	return NULL;
 }
 
-const struct got_error *
-got_object_qid_alloc(struct got_object_qid **qid, struct got_object_id *id)
-{
-	*qid = calloc(1, sizeof(**qid));
-	if (*qid == NULL)
-		return got_error_from_errno("calloc");
-
-	memcpy(&(*qid)->id, id, sizeof((*qid)->id));
-	return NULL;
-}
-
-const struct got_error *
-got_object_id_queue_copy(const struct got_object_id_queue *src,
-    struct got_object_id_queue *dest)
-{
-	const struct got_error *err;
-	struct got_object_qid *qid;
-
-	STAILQ_FOREACH(qid, src, entry) {
-		struct got_object_qid *new;
-		/*
-		 * Deep-copy the object ID only. Let the caller deal
-		 * with setting up the new->data pointer if needed.
-		 */
-		err = got_object_qid_alloc(&new, &qid->id);
-		if (err) {
-			got_object_id_queue_free(dest);
-			return err;
-		}
-		STAILQ_INSERT_TAIL(dest, new, entry);
-	}
-
-	return NULL;
-}
-
 int
 got_object_tree_get_nentries(struct got_tree_object *tree)
 {
