@@ -34,11 +34,11 @@
 #include "got_object.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
 #include "got_lib_object_parse.h"
 #include "got_lib_privsep.h"
-#include "got_lib_sha1.h"
 
 static volatile sig_atomic_t sigint_received;
 
@@ -84,6 +84,7 @@ main(int argc, char *argv[])
 		struct imsg imsg;
 		struct got_tag_object *tag = NULL;
 		struct got_object_id expected_id;
+		int fd = -1;
 
 		if (sigint_received) {
 			err = got_error(GOT_ERR_CANCELLED);
@@ -112,19 +113,20 @@ main(int argc, char *argv[])
 		}
 		memcpy(&expected_id, imsg.data, sizeof(expected_id));
 
-		if (imsg.fd == -1) {
+		fd = imsg_get_fd(&imsg);
+		if (fd == -1) {
 			err = got_error(GOT_ERR_PRIVSEP_NO_FD);
 			goto done;
 		}
 
 		/* Always assume file offset zero. */
-		err = got_object_read_tag(&tag, imsg.fd, &expected_id, 0);
+		err = got_object_read_tag(&tag, fd, &expected_id, 0);
 		if (err)
 			goto done;
 
 		err = got_privsep_send_tag(&ibuf, tag);
 done:
-		if (imsg.fd != -1 && close(imsg.fd) == -1 && err == NULL)
+		if (fd != -1 && close(fd) == -1 && err == NULL)
 			err = got_error_from_errno("close");
 		imsg_free(&imsg);
 		if (err)
