@@ -190,8 +190,15 @@ EOF
 		return 1
 	fi
 
-	echo "got: refs/heads/master: fetch and rebase required" \
-		> $testroot/stderr.expected
+	echo -n "got: refs/heads/master: " > $testroot/stderr.expected
+	echo -n "branch on server has a different ancestry; " \
+		>> $testroot/stderr.expected
+	echo -n "either fetch changes from server and then " \
+		>> $testroot/stderr.expected
+	echo -n "rebase or merge local branch before sending, or " \
+		>> $testroot/stderr.expected
+	echo "ignore ancestry with send -f (can lead to data loss on server)" \
+		>> $testroot/stderr.expected
 	cmp -s $testroot/stderr $testroot/stderr.expected
 	ret=$?
 	if [ $ret -ne 0 ]; then
@@ -1726,19 +1733,19 @@ EOF
 }
 
 test_parseargs "$@"
-run_test test_send_basic
-run_test test_send_rebase_required
-run_test test_send_rebase_required_overwrite
-run_test test_send_merge_commit
-run_test test_send_delete
-run_test test_send_clone_and_send
-run_test test_send_tags
-run_test test_send_tag_of_deleted_branch
-run_test test_send_new_branch
-run_test test_send_all_branches
-run_test test_send_to_empty_repo
-run_test test_send_and_fetch_config
-run_test test_send_config
-run_test test_send_gitconfig
-run_test test_send_rejected
-run_test test_send_basic_http
+run_test test_send_basic			no-sha256
+run_test test_send_rebase_required		no-sha256
+run_test test_send_rebase_required_overwrite	no-sha256
+run_test test_send_merge_commit			no-sha256
+run_test test_send_delete			no-sha256
+run_test test_send_clone_and_send		no-sha256
+run_test test_send_tags				no-sha256
+run_test test_send_tag_of_deleted_branch	no-sha256
+run_test test_send_new_branch			no-sha256
+run_test test_send_all_branches			no-sha256
+run_test test_send_to_empty_repo		no-sha256
+run_test test_send_and_fetch_config		no-sha256
+run_test test_send_config			no-sha256
+run_test test_send_gitconfig			no-sha256
+run_test test_send_rejected			no-sha256
+run_test test_send_basic_http			no-sha256
